//
// Created by robin on 5/10/16.
//

#include <pcap.h>
#include "Pcap.h"

struct CustomPcapData {
    Pcap* pcap;
    void* data;
    PcapCallback callback;
};

Pcap::Pcap(const std::string &device)
    : _device(device)
    , _pcap(0)
    , _isInLoop(false)
{
}

Pcap::~Pcap()
{
    close();
}

ResultBool Pcap::init(int snaplen, bool promisc, int timeout)
{
    char errbuf[PCAP_ERRBUF_SIZE];
    _pcap = pcap_open_live(_device.c_str(), snaplen, promisc, timeout, errbuf);
    if (!_pcap) {
        return ResultBool::error(errbuf);
    }
    return ResultBool::ok(true);
}

ResultString Pcap::getDefaultDevice()
{
    char errbuf[PCAP_ERRBUF_SIZE];
    char* dev = pcap_lookupdev(errbuf);
    if (!dev) {
        return ResultString::error(errbuf);
    }
    return ResultString::ok(dev);
}

ResultBool Pcap::run(PcapCallback callback, void* data, int count)
{
    if (!_pcap) {
        return ResultBool::error("Pcap has not been initialized");
    }
    CustomPcapData customData;
    customData.pcap = this;
    customData.data = data;
    customData.callback = callback;
    _isInLoop = true;
    int res = pcap_loop(_pcap, count, [](u_char *args, const struct pcap_pkthdr *header, const u_char *packet) {
        CustomPcapData* customData = (CustomPcapData*)args;
        customData->pcap->onPacket(args, header, packet);
    }, (u_char*)&customData);
    _isInLoop = false;
    if (res == -1) {
        return ResultBool::error(getPcapLastError());
    }
    return ResultBool::ok(true);
}

void Pcap::onPacket(u_char* args, const struct pcap_pkthdr* header, const u_char* packet)
{
    CustomPcapData* customData = (CustomPcapData*)args;
    if (!customData->callback(customData->data, header->caplen, (const char*)packet)) {
        stop();
    }
}

std::string Pcap::getPcapLastError() const
{
    return pcap_geterr(_pcap);
}

void Pcap::stop()
{
    if (_pcap && _isInLoop) {
        pcap_breakloop(_pcap);
    }
}

void Pcap::close()
{
    if (_pcap) {
        pcap_close(_pcap);
        _pcap = 0;
    }
}

ResultBool Pcap::send(void *data, unsigned int len)
{
    if (!_pcap) {
        return ResultBool::error("Pcap has not been initialized");
    }
    int res = pcap_sendpacket(_pcap, (const u_char*)data, len);
    if (res == -1) {
        return ResultBool::error(getPcapLastError());
    }
    return ResultBool::ok(true);
}
